@@ -4,89 +4,16 @@ import imp
 import sets
 import itertools
 
+from zope.interface import implements
+
 from twisted.internet import reactor, protocol, defer, error
 from twisted.python import log, util, reflect
 from twisted.protocols import amp
 
+from ampoule import iampoule
+
 gen = itertools.count()
 
-BOOTSTRAP = """\
-import sys
-
-def main(reactor, ampChildPath):
-    from twisted.application import reactors
-    reactors.installReactor(reactor)
-    
-    from twisted.python import log
-    log.startLogging(sys.stderr)
-
-    from twisted.internet import reactor, stdio
-    from twisted.python import reflect
-
-    ampChild = reflect.namedAny(ampChildPath)
-    stdio.StandardIO(ampChild(), 3, 4)
-    reactor.run()
-main(sys.argv[1], sys.argv[2])
-"""
-
-def _checkRoundTrip(obj): 
-    """
-    Make sure that an object will properly round-trip through 'qual' and
-    'namedAny'.
-
-    Raise a L{RuntimeError} if they aren't.
-    """
-    tripped = reflect.namedAny(reflect.qual(obj))
-    if tripped is not obj:
-        raise RuntimeError("importing %r is not the same as %r" %
-                           (reflect.qual(obj), obj))
-
-def startAMPProcess(ampChild, *args, **kwargs):
-    """
-    @param ampChild: a L{ampoule.child.AMPChild} subclass.
-    @type ampChild: L{ampoule.child.AMPChild}
-    
-    @param ampParent: an L{amp.AMP} subclass that implements the parent
-                      protocol for this process pool
-    @type ampParent: L{amp.AMP}
-    
-    @param args: a tuple of arguments that will be passed to the
-                 subprocess
-    @param kwargs: a dictionary that contains extra arguments for the
-                   spawnProcess call.
-    
-    @param childReactor: a string that sets the reactor for child
-                         processes
-    @type childReactor: L{str}
-    """
-    _checkRoundTrip(ampChild)
-    fullPath = reflect.qual(ampChild)
-    ampParent = kwargs.pop('ampParent', None)
-    if ampParent is None:
-        ampParent = amp.AMP
-    childReactor = kwargs.pop('childReactor', None)
-    if childReactor is None:
-        childReactor = "select"
-    prot = AMPConnector(ampParent())
-    return startProcess(prot, childReactor, fullPath, *args, **kwargs)
-
-def startProcess(prot, *args, **kwargs):
-    """
-    @param prot: a L{protocol.ProcessProtocol} subclass
-    @type prot: L{protocol.ProcessProtocol}
-    
-    @return: a tuple of the child process and the deferred finished.
-             finished triggers when the subprocess dies for any reason.
-    """
-    spawnProcess(prot, tuple(args), **kwargs)
-    
-    # XXX: we could wait for startup here, but ... is there really any
-    # reason to?  the pipe should be ready for writing.  The subprocess
-    # might not start up properly, but then, a subprocess might shut down
-    # at any point too. So we just return amp and have this piece to be
-    # synchronous.
-    return prot.amp, prot.finished
-
 class AMPConnector(protocol.ProcessProtocol):
     """
     A L{ProcessProtocol} subclass that can understand and speak AMP.
@@ -154,9 +81,130 @@ class AMPConnector(protocol.ProcessProtocol):
             return
         self.finished.errback(status)
 
-def spawnProcess(processProtocol, args=(), env={},
+BOOTSTRAP = """\
+import sys
+
+def main(reactor, ampChildPath):
+    from twisted.application import reactors
+    reactors.installReactor(reactor)
+    
+    from twisted.python import log
+    log.startLogging(sys.stderr)
+
+    from twisted.internet import reactor, stdio
+    from twisted.python import reflect
+
+    ampChild = reflect.namedAny(ampChildPath)
+    stdio.StandardIO(ampChild(), 3, 4)
+    reactor.run()
+main(sys.argv[-2], sys.argv[-1])
+"""
+
+class ProcessStarter(object):
+    
+    implements(iampoule.IStarter)
+    
+    connectorFactory = AMPConnector
+    def __init__(self, bootstrap=BOOTSTRAP, args=(), env={},
                  path=None, uid=None, gid=None, usePTY=0,
-                 packages=(), bootstrap=BOOTSTRAP):
+                 packages=(), childReactor="select"):
+        """
+        @param bootstrap: Startup code for the child process
+        @type  bootstrap: C{str}
+        
+        @param args: Arguments that should be supplied to every child
+                     created.
+        @type args: C{tuple} of C{str}
+        
+        @param env: Environment variables that should be present in the
+                    child environment
+        @type env: C{dict}
+        
+        @param path: Path in which to run the child
+        @type path: C{str}
+        
+        @param uid: if defined, the uid used to run the new process.
+        @type uid: C{int}
+        
+        @param gid: if defined, the gid used to run the new process.
+        @type gid: C{int}
+        
+        @param usePTY: Should the child processes use PTY processes
+        @type usePTY: 0 or 1
+        
+        @param packages: A tuple of packages that should be guaranteed
+                         to be importable in the child processes
+        @type packages: C{tuple} of C{str}
+        
+        @param childReactor: a string that sets the reactor for child
+                             processes
+        @type childReactor: C{str}
+        """
+        self.bootstrap = bootstrap
+        self.args = args
+        self.env = env
+        self.path = path
+        self.uid = uid
+        self.gid = gid
+        self.usePTY = usePTY
+        self.packages = packages
+        self.childReactor = childReactor
+
+    def _checkRoundTrip(self, obj):
+        """
+        Make sure that an object will properly round-trip through 'qual' and
+        'namedAny'.
+
+        Raise a L{RuntimeError} if they aren't.
+        """
+        tripped = reflect.namedAny(reflect.qual(obj))
+        if tripped is not obj:
+            raise RuntimeError("importing %r is not the same as %r" %
+                               (reflect.qual(obj), obj))
+
+    def startAMPProcess(self, ampChild, ampParent=None):
+        """
+        @param ampChild: a L{ampoule.child.AMPChild} subclass.
+        @type ampChild: L{ampoule.child.AMPChild}
+    
+        @param ampParent: an L{amp.AMP} subclass that implements the parent
+                          protocol for this process pool
+        @type ampParent: L{amp.AMP}
+        """
+        self._checkRoundTrip(ampChild)
+        fullPath = reflect.qual(ampChild)
+        if ampParent is None:
+            ampParent = amp.AMP
+        prot = self.connectorFactory(ampParent())
+        
+        return self.startPythonProcess(prot, self.childReactor, fullPath)
+
+
+    def startPythonProcess(self, prot, *args):
+        """
+        @param prot: a L{protocol.ProcessProtocol} subclass
+        @type prot: L{protocol.ProcessProtocol}
+    
+        @param args: a tuple of arguments that will be added after the
+                     ones in L{self.args} to start the child process.
+    
+        @return: a tuple of the child process and the deferred finished.
+                 finished triggers when the subprocess dies for any reason.
+        """
+        spawnProcess(prot, self.bootstrap, self.args+args, env=self.env,
+                     path=self.path, uid=self.uid, gid=self.gid,
+                     usePTY=self.usePTY, packages=self.packages)
+    
+        # XXX: we could wait for startup here, but ... is there really any
+        # reason to?  the pipe should be ready for writing.  The subprocess
+        # might not start up properly, but then, a subprocess might shut down
+        # at any point too. So we just return amp and have this piece to be
+        # synchronous.
+        return prot.amp, prot.finished
+
+def spawnProcess(processProtocol, bootstrap, args=(), env={},
+                 path=None, uid=None, gid=None, usePTY=0,
+                 packages=()):
     env = env.copy()
 
     pythonpath = []
