@@ -164,18 +164,20 @@ class TestAMPConnector(unittest.TestCase):
         Test that startProcess actually starts a subprocess and that
         it receives data back from the process through AMP.
         """
-        STRING = "ciao"
         s = sio()
         a = FakeAMP(s)
+        STRING = "ciao"
         BOOT = """\
 import sys, os
 def main(arg):
     os.write(4, arg)
 main(sys.argv[1])
 """
-        amp, finished = main.startProcess(main.AMPConnector(a),
-                                          STRING, bootstrap=BOOT,
-                                          packages=("ampoule", "twisted"))
+        starter = main.ProcessStarter(bootstrap=BOOT,
+                                      args=(STRING,),
+                                      packages=("twisted", "ampoule"))
+
+        amp, finished = starter.startPythonProcess(main.AMPConnector(a))
         def _eb(reason):
             print reason
         finished.addErrback(_eb)
@@ -186,29 +188,54 @@ main(sys.argv[1])
         Test failing subprocesses and the way they terminate and preserve
         failing information.
         """
-        STRING = "ciao"
         s = sio()
         a = FakeAMP(s)
+        STRING = "ciao"
         BOOT = """\
 import sys
 def main(arg):
     raise Exception(arg)
 main(sys.argv[1])
 """
-        ready, finished = main.startProcess(main.AMPConnector(a),
-                                            STRING, bootstrap=BOOT,
-                                            packages=("ampoule", "twisted"))
+        starter = main.ProcessStarter(bootstrap=BOOT, args=(STRING,), packages=("twisted", "ampoule"))
+        ready, finished = starter.startPythonProcess(main.AMPConnector(a), "I'll be ignored")
+        
         self.assertFailure(finished, error.ProcessTerminated)
         finished.addErrback(lambda reason: self.assertEquals(reason.getMessage(), STRING))
         return finished
 
+    def test_env_setting(self):
+        """
+        Test that and environment variable passed to the process starter
+        is correctly passed to the child process.
+        """
+        s = sio()
+        a = FakeAMP(s)
+        STRING = "ciao"
+        BOOT = """\
+import sys, os
+def main():
+    os.write(4, os.getenv("FOOBAR"))
+main()
+"""
+        starter = main.ProcessStarter(bootstrap=BOOT,
+                                      packages=("twisted", "ampoule"),
+                                      env={"FOOBAR": STRING})
+        amp, finished = starter.startPythonProcess(main.AMPConnector(a), "I'll be ignored")
+        def _eb(reason):
+            print reason
+        finished.addErrback(_eb)
+        return finished.addCallback(lambda _: self.assertEquals(s.getvalue(), STRING))
+
     def test_startAMPProcess(self):
         """
         Test that you can start an AMP subprocess and that it correctly
         accepts commands and correctly answers them.
         """
         STRING = "ciao"
-        c, finished = main.startAMPProcess(child.AMPChild, packages=('ampoule', 'twisted'))
+        
+        starter = main.ProcessStarter(packages=("twisted", "ampoule"))
+        c, finished = starter.startAMPProcess(child.AMPChild)
         c.callRemote(commands.Echo, data=STRING
            ).addCallback(lambda response:
                 self.assertEquals(response['response'], STRING)
@@ -228,7 +255,9 @@ main(sys.argv[1])
                 return {'response': DATA+APPEND}
             Pong.responder(pong)
         
-        subp, finished = main.startAMPProcess(Child, ampParent=Parent, packages=('ampoule', 'twisted'))
+        starter = main.ProcessStarter(packages=("twisted", "ampoule"))
+
+        subp, finished = starter.startAMPProcess(ampChild=Child, ampParent=Parent)
         subp.callRemote(Ping, data=DATA
            ).addCallback(lambda response:
                 self.assertEquals(response['response'], DATA+APPEND)
@@ -243,9 +272,12 @@ main(sys.argv[1])
         class Child(child.AMPChild):
             pass
         
-        self.assertRaises(RuntimeError, main.startAMPProcess, Child, packages=('ampoule', 'twisted'))
+        starter = main.ProcessStarter(packages=("twisted", "ampoule"))
+
+        self.assertRaises(RuntimeError, starter.startAMPProcess, ampChild=Child)
 
 class TestProcessPool(unittest.TestCase):
+        
     def test_startStopWorker(self):
         """
         Test that starting and stopping a worker keeps the state of
@@ -340,7 +372,7 @@ class TestProcessPool(unittest.TestCase):
         """
         Test that a failing child process is immediately restarted.
         """
-        pp = pool.ProcessPool(BadChild, min=1)
+        pp = pool.ProcessPool(ampChild=BadChild, min=1)
         STRING = "DATA"
         
         def _checks(_):
@@ -368,7 +400,7 @@ class TestProcessPool(unittest.TestCase):
                 return {'response': DATA+APPEND}
             Pong.responder(pong)
         
-        pp = pool.ProcessPool(Child, ampParent=Parent)
+        pp = pool.ProcessPool(ampChild=Child, ampParent=Parent)
         def _checks(_):
             return pp.doWork(Ping, data=DATA
                        ).addCallback(lambda response:
@@ -392,12 +424,12 @@ class TestProcessPool(unittest.TestCase):
         """
         Test that busy and ready lists are correctly maintained.
         """
-        pp = pool.ProcessPool(WaitingChild)
+        pp = pool.ProcessPool(ampChild=WaitingChild)
         
         DATA = "foobar"
 
         def _checks(_):
-            d = pp.doWork(First, data=DATA)
+            d = pp.callRemote(First, data=DATA)
             self.assertEquals(pp.started, True)
             self.assertEquals(pp.finished, False)
             self.assertEquals(len(pp.processes), pp.min)
@@ -418,7 +450,7 @@ class TestProcessPool(unittest.TestCase):
         Test that the pool grows over time until it reaches max processes.
         """
         MAX = 5
-        pp = pool.ProcessPool(WaitingChild, min=1, max=MAX)
+        pp = pool.ProcessPool(ampChild=WaitingChild, min=1, max=MAX)
 
         def _checks(_):
             self.assertEquals(pp.started, True)
@@ -450,7 +482,7 @@ class TestProcessPool(unittest.TestCase):
         MAX = 5
         MIN = 1
         IDLE = 1
-        pp = pool.ProcessPool(WaitingChild, min=MIN, max=MAX, maxIdle=IDLE)
+        pp = pool.ProcessPool(ampChild=WaitingChild, min=MIN, max=MAX, maxIdle=IDLE)
                 
         def _checks(_):
             self.assertEquals(pp.started, True)
@@ -501,7 +533,7 @@ class TestProcessPool(unittest.TestCase):
         MAX = 1
         MIN = 1
         RECYCLE_AFTER = 1
-        pp = pool.ProcessPool(PidChild, min=MIN, max=MAX, recycleAfter=RECYCLE_AFTER)
+        pp = pool.ProcessPool(ampChild=PidChild, min=MIN, max=MAX, recycleAfter=RECYCLE_AFTER)
         
         def _checks(_):
             self.assertEquals(pp.started, True)
@@ -533,7 +565,7 @@ class TestProcessPool(unittest.TestCase):
         MIN = 1
         RECYCLE_AFTER = 10
         CALLS = 60
-        pp = pool.ProcessPool(PidChild, min=MIN, max=MAX, recycleAfter=RECYCLE_AFTER)
+        pp = pool.ProcessPool(ampChild=PidChild, min=MIN, max=MAX, recycleAfter=RECYCLE_AFTER)
         
         def _check(results):
             s = set()
@@ -556,7 +588,7 @@ class TestProcessPool(unittest.TestCase):
         MAX = 1
         MIN = 1
         RECYCLE_AFTER = 0
-        pp = pool.ProcessPool(PidChild, min=MIN, max=MAX, recycleAfter=RECYCLE_AFTER)
+        pp = pool.ProcessPool(ampChild=PidChild, min=MIN, max=MAX, recycleAfter=RECYCLE_AFTER)
         
         def _checks(_):
             self.assertEquals(pp.started, True)
@@ -592,14 +624,14 @@ class TestProcessPool(unittest.TestCase):
         SECOND = "poll"
         
         def checkDefault():
-            pp = pool.ProcessPool(ReactorChild, min=MIN, max=MAX, childReactor=FIRST)
+            pp = pool.ProcessPool(starter=main.ProcessStarter(childReactor=FIRST), ampChild=ReactorChild, min=MIN, max=MAX)
             pp.start()
             return pp.doWork(Reactor
                 ).addCallback(self.assertEquals, {'classname': "SelectReactor"}
                 ).addCallback(lambda _: pp.stop())
             
         def checkPool(_):
-            pp = pool.ProcessPool(ReactorChild, min=MIN, max=MAX, childReactor=SECOND)
+            pp = pool.ProcessPool(starter=main.ProcessStarter(childReactor=SECOND), ampChild=ReactorChild, min=MIN, max=MAX)
             pp.start()
             return pp.doWork(Reactor
                 ).addCallback(self.assertEquals, {'classname': "PollReactor"}
@@ -618,7 +650,7 @@ class TestProcessPool(unittest.TestCase):
         actually don't have any problems.
         """
         DATA = "hello"
-        pp = pool.ProcessPool(NoResponseChild, min=1, max=1)
+        pp = pool.ProcessPool(ampChild=NoResponseChild, min=1, max=1)
 
         def _check(_):
             return pp.doWork(GetResponse
